package ru.dbotthepony.mc.otm.datagen.items

import net.minecraft.resources.ResourceLocation
import net.minecraft.world.item.Item
import net.minecraftforge.client.model.generators.ItemModelProvider
import net.minecraftforge.data.event.GatherDataEvent
import org.apache.logging.log4j.LogManager
import ru.dbotthepony.mc.otm.datagen.DataGen
import ru.dbotthepony.mc.otm.core.registryName

private data class SimpleItemModel(val item: String, val path: ResourceLocation) {
	val traceback = IllegalArgumentException("Failed to register model")
}

class MatteryItemModelProvider(event: GatherDataEvent) : ItemModelProvider(event.generator, DataGen.MOD_ID, event.existingFileHelper) {
	private val generated = ArrayList<SimpleItemModel>()
	private val handheld = ArrayList<SimpleItemModel>()
	private val delegates = ArrayList<SimpleItemModel>()

	override fun registerModels() {
		for (obj in generated) {
			val (item, path) = obj

			try {
				withExistingParent(item, GENERATED).texture("layer0", path)
			} catch(err: Throwable) {
				LOGGER.error(err, obj.traceback)
				throw err
			}
		}

		for (obj in handheld) {
			val (item, path) = obj

			try {
				withExistingParent(item, HANDHELD).texture("layer0", path)
			} catch(err: Throwable) {
				LOGGER.error(err, obj.traceback)
				throw err
			}
		}

		for (obj in delegates) {
			val (item, path) = obj

			try {
				withExistingParent(item, path)
			} catch(err: Throwable) {
				LOGGER.error(err, obj.traceback)
				throw err
			}
		}
	}

	fun block(item: Item) = delegates.add(SimpleItemModel(item.registryName!!.path, ResourceLocation(DataGen.MOD_ID, "block/${item.registryName!!.path}")))
	fun block(item: Item, path: String) = delegates.add(SimpleItemModel(item.registryName!!.path, ResourceLocation(DataGen.MOD_ID, "block/$path")))
	fun blocks(vararg items: Item) = items.forEach(this::block)
	fun blocks(items: List<Item>) = items.forEach(this::block)

	fun generated(item: Item, texture: ResourceLocation) {
		generated.add(SimpleItemModel(item.registryName!!.path, texture))
	}

	fun handheld(item: Item, texture: ResourceLocation) {
		handheld.add(SimpleItemModel(item.registryName!!.path, texture))
	}

	fun generated(vararg items: Item) = items.forEach { generated(it, ResourceLocation(DataGen.MOD_ID, "item/${it.registryName!!.path}")) }
	fun generated(items: List<Item>) = items.forEach { generated(it, ResourceLocation(DataGen.MOD_ID, "item/${it.registryName!!.path}")) }
	fun generatedBlock(vararg items: Item) = items.forEach { generated(it, ResourceLocation(DataGen.MOD_ID, "block/${it.registryName!!.path}")) }
	fun generatedBlockDecorative(vararg items: Item) = items.forEach { generated(it, ResourceLocation(DataGen.MOD_ID, "block/decorative/${it.registryName!!.path}")) }
	fun handheld(vararg items: Item) = items.forEach { handheld(it, ResourceLocation(DataGen.MOD_ID, "item/${it.registryName!!.path}")) }
	fun handheld(items: List<Item>) = items.forEach { handheld(it, ResourceLocation(DataGen.MOD_ID, "item/${it.registryName!!.path}")) }

	fun generated(item: Item, prefix: String) = generated(item, ResourceLocation(DataGen.MOD_ID, "item/${prefix}${item.registryName!!.path}"))
	fun handheld(item: Item, prefix: String) = handheld(item, ResourceLocation(DataGen.MOD_ID, "item/${prefix}${item.registryName!!.path}"))

	fun component(item: Item) = generated(item, "component/")
	fun components(vararg items: Item) = items.forEach(this::component)
	fun components(items: List<Item>) = items.forEach(this::component)
	fun resource(item: Item) = generated(item, "resources/")
	fun resources(vararg items: Item) = items.forEach(this::resource)

	fun generatedTiered(items: Array<out Item>, prefix: String) {
		var i = 0

		for (item in items) {
			generated(item, ResourceLocation(DataGen.MOD_ID, "item/$prefix$i"))
			i++
		}
	}

	fun generatedTiered(items: List<Item>, prefix: String) {
		var i = 0

		for (item in items) {
			generated(item, ResourceLocation(DataGen.MOD_ID, "item/$prefix$i"))
			i++
		}
	}

	companion object {
		private val GENERATED = ResourceLocation("minecraft", "item/generated")
		private val HANDHELD = ResourceLocation("minecraft", "item/handheld")
		private val LOGGER = LogManager.getLogger()
	}
}
