package ru.dbotthepony.mc.otm.item

import net.minecraft.ChatFormatting
import net.minecraft.nbt.CompoundTag
import net.minecraft.network.chat.Component
import net.minecraft.util.Mth
import net.minecraft.world.item.Item
import net.minecraft.world.item.ItemStack
import net.minecraft.world.item.Rarity
import net.minecraft.world.item.TooltipFlag
import net.minecraft.world.level.Level
import net.minecraftforge.common.capabilities.ICapabilityProvider
import ru.dbotthepony.mc.otm.OverdriveThatMatters
import ru.dbotthepony.mc.otm.capability.*
import ru.dbotthepony.mc.otm.core.ImpreciseFraction
import ru.dbotthepony.mc.otm.core.TranslatableComponent
import ru.dbotthepony.mc.otm.core.formatPower
import ru.dbotthepony.mc.otm.core.ifPresentK

class BatteryItem : Item {
	private inner class BatteryMatteryCapability(stack: ItemStack)
		: EnergyCapacitorItem(stack, this@BatteryItem.storage, this@BatteryItem.receive, this@BatteryItem.extract) {
		override fun extractEnergyInner(howMuch: ImpreciseFraction, simulate: Boolean): ImpreciseFraction {
			if (isCreative) return howMuch
			return super.extractEnergyInner(howMuch, simulate)
		}

		override fun receiveEnergyInner(howMuch: ImpreciseFraction, simulate: Boolean): ImpreciseFraction {
			if (isCreative) return howMuch
			return super.receiveEnergyInner(howMuch, simulate)
		}

		override val missingPower: ImpreciseFraction get() {
			return if (isCreative) ImpreciseFraction.LONG_MAX_VALUE else super.missingPower
		}

		override var batteryLevel: ImpreciseFraction
			get() { return if (isCreative) ImpreciseFraction.LONG_MAX_VALUE else super.batteryLevel }
			set(value) { super.batteryLevel = value }
	}

	private val isCreative: Boolean
	val storage: ImpreciseFraction
	val receive: ImpreciseFraction
	val extract: ImpreciseFraction

	constructor(storage: ImpreciseFraction, receive: ImpreciseFraction, extract: ImpreciseFraction) : super(
		Properties().stacksTo(1).tab(OverdriveThatMatters.INSTANCE.CREATIVE_TAB)
	) {
		isCreative = false
		this.storage = storage
		this.receive = receive
		this.extract = extract
	}

	constructor() : super(Properties().stacksTo(1).rarity(Rarity.EPIC).tab(OverdriveThatMatters.INSTANCE.CREATIVE_TAB)) {
		isCreative = true
		storage = ImpreciseFraction.LONG_MAX_VALUE
		receive = ImpreciseFraction.LONG_MAX_VALUE
		extract = ImpreciseFraction.LONG_MAX_VALUE
	}

	override fun isBarVisible(p_150899_: ItemStack): Boolean {
		if (isCreative)
			return false

		return p_150899_.matteryEnergy != null
	}

	override fun getBarWidth(p_150900_: ItemStack): Int {
		if (isCreative)
			return 13

		return p_150900_.matteryEnergy?.getBarWidth() ?: super.getBarWidth(p_150900_)
	}

	override fun getBarColor(p_150901_: ItemStack): Int {
		if (isCreative)
			return 0

		return p_150901_.matteryEnergy?.getBarColor() ?: super.getBarColor(p_150901_)
	}

	override fun appendHoverText(
		stack: ItemStack,
		p_41422_: Level?,
		p_41423_: MutableList<Component>,
		p_41424_: TooltipFlag
	) {
		super.appendHoverText(stack, p_41422_, p_41423_, p_41424_)

		if (isCreative) {
			p_41423_.add(INFINITE_STORAGE)
			p_41423_.add(TranslatableComponent("otm.item.power.infinite.throughput").withStyle(ChatFormatting.GRAY))
		} else {
			stack.getCapability(MatteryCapability.ENERGY).ifPresentK {
				p_41423_.add(
					TranslatableComponent(
						"otm.item.power.normal.storage",
						it.batteryLevel.formatPower(),
						it.maxBatteryLevel.formatPower()
					).withStyle(ChatFormatting.GRAY)
				)
			}

			p_41423_.add(TranslatableComponent(
				"otm.item.power.normal.throughput",
				receive.formatPower(),
				extract.formatPower()
			).withStyle(ChatFormatting.GRAY))
		}
	}

	override fun initCapabilities(stack: ItemStack, nbt: CompoundTag?): ICapabilityProvider {
		return BatteryMatteryCapability(stack)
	}

	companion object {
		private val INFINITE_STORAGE: Component =
			TranslatableComponent("otm.item.power.infinite.storage").withStyle(ChatFormatting.GRAY)
	}
}