package ru.dbotthepony.mc.otm.menu.widget

import net.minecraft.network.FriendlyByteBuf
import net.minecraftforge.network.NetworkEvent
import ru.dbotthepony.mc.otm.menu.MatteryMenu
import ru.dbotthepony.mc.otm.network.*
import java.math.BigDecimal
import java.util.function.Supplier

class NumberPlayerInputPacket(val id: Int, val value: BigDecimal) : MatteryPacket {
	override fun play(context: Supplier<NetworkEvent.Context>) {
		context.get().packetHandled = true

		context.get().enqueueWork {
			((context.get().sender?.containerMenu as? MatteryMenu)?.getWidget(id) as? NumberPlayerInputWidget)?.userInput(value)
		}
	}

	override fun write(buff: FriendlyByteBuf) {
		buff.writeInt(id)
		buff.writeDecimal(value)
	}

	companion object {
		fun read(buff: FriendlyByteBuf): NumberPlayerInputPacket {
			return NumberPlayerInputPacket(buff.readInt(), buff.readDecimal())
		}
	}
}

class NumberPlayerInputWidget(menu: MatteryMenu) : AbstractWidget(menu) {
	var value by menu.mSynchronizer.bigDecimal()

	var supplier: (() -> BigDecimal)? = null
	var consumer: ((BigDecimal) -> Unit)? = null
	var minValue: BigDecimal? = null
	var maxValue: BigDecimal? = null

	fun withSupplier(func: () -> BigDecimal): NumberPlayerInputWidget {
		supplier = func
		return this
	}

	fun withConsumer(func: (BigDecimal) -> Unit): NumberPlayerInputWidget {
		consumer = func
		return this
	}

	fun asClient(): NumberPlayerInputWidget {
		supplier = null

		consumer = {
			MenuNetworkChannel.sendToServer(NumberPlayerInputPacket(slotID, it))
		}

		return this
	}

	fun userInput(newValue: BigDecimal) {
		val consumer = consumer

		if (consumer != null) {
			@Suppress("name_shadowing")
			var newValue = newValue

			if (minValue != null) {
				newValue = newValue.max(minValue!!)
			}

			if (maxValue != null) {
				newValue = newValue.min(maxValue!!)
			}

			consumer.invoke(newValue)
			updateServer()
		}
	}

	override fun updateServer() {
		val supplier = supplier

		if (supplier != null) {
			value = supplier.invoke()
		}
	}
}
