package ru.dbotthepony.mc.otm.tests

import org.junit.jupiter.api.DisplayName
import org.junit.jupiter.api.Test
import ru.dbotthepony.mc.otm.core.ImpreciseFraction

object ImpreciseFractionTests {
	@Test
	@DisplayName("ImpreciseFraction comparison")
	fun comparison() {
		check(ImpreciseFraction(642, 0.43774) > ImpreciseFraction(641, 0.43774)) { "must be bigger" }
		check(ImpreciseFraction(642, 0.43774) > ImpreciseFraction(-641, 0.43774)) { "must be bigger" }
		check(ImpreciseFraction(0) == ImpreciseFraction(0)) { "integer notation" }
		check(ImpreciseFraction(0.1) > ImpreciseFraction(0)) { "must be bigger" }
		check(ImpreciseFraction(-0.1) < ImpreciseFraction(0)) { "must be lesser" }
		check(ImpreciseFraction(-1, -0.1) < ImpreciseFraction(-1)) { "must be lesser" }
		check(ImpreciseFraction(0.1) == ImpreciseFraction(0.1)) { "double notation" }
		check(ImpreciseFraction("0.1") == ImpreciseFraction("0.1")) { "string notation" }
	}

	@Test
	@DisplayName("ImpreciseFraction mathematical operations")
	fun math() {
		check((ImpreciseFraction(1) + ImpreciseFraction(2)) == ImpreciseFraction(3)) { "1 + 2 != 3" }
		check((ImpreciseFraction(0, 0.5) + ImpreciseFraction(0, 0.5)) == ImpreciseFraction(1)) { "0.5 + 0.5 != 1" }
		check((ImpreciseFraction(0, 0.5) + ImpreciseFraction(0, -0.5)) == ImpreciseFraction(0)) { "0.5 + -0.5 != 1" }
		check((ImpreciseFraction(0, 0.5) - ImpreciseFraction(0, -0.5)) == ImpreciseFraction(1)) { "0.5 - -0.5 != 1" }
		check((ImpreciseFraction(0, 0.3) - ImpreciseFraction(1, 0.2)) == ImpreciseFraction(0, -0.9)) { "0.3 - 1.2 != -0.9" }
		check((ImpreciseFraction(0, 0.3) * ImpreciseFraction(0, 0.3)) == ImpreciseFraction(0, 0.09)) { "0.3 * 0.3 != 0.9 ${ImpreciseFraction(0, 0.3) * ImpreciseFraction(0, 0.3)}" }
		check((ImpreciseFraction(2, 0.3) * ImpreciseFraction(2, 0.3)) == ImpreciseFraction(5, 0.29)) { "2.3 * 2.3 != 5.29 ${ImpreciseFraction(2, 0.3) * ImpreciseFraction(2, 0.3)}" }
		check((ImpreciseFraction(4) / ImpreciseFraction(2)) == ImpreciseFraction(2)) { "4 / 2 != 2" }
		check((ImpreciseFraction(6) / ImpreciseFraction(2)) == ImpreciseFraction(3)) { "6 / 2 != 2" }
		check((ImpreciseFraction(1) / ImpreciseFraction(0, 0.25)) == ImpreciseFraction(4)) { "1 / 0.25 != 4" }
	}

	@Test
	@DisplayName("ImpreciseFraction store/load")
	fun storeLoad() {
		run {
			val f = ImpreciseFraction(4, 0.28)
			val loaded = ImpreciseFraction.fromByteArray(f.toByteArray())
			check(f == loaded) { "$f != $loaded" }
		}

		run {
			val f = ImpreciseFraction(32748293658335L, 0.3472302174)
			val loaded = ImpreciseFraction.fromByteArray(f.toByteArray())
			check(f == loaded) { "$f != $loaded" }
		}
	}
}
